% This program implements  the Case Study
% CS_SSD_Portfolio_Optimization with mpsg_solver 

clc;
clear;

%Load problem data
load('.\SSD_Portfolio_Optimization_data_Toolbox.mat');

%Save variables from structure toolboxstruc_arr to Workspace
tbpsg_export_to_workspace(toolboxstruc_arr);

J=size(toolboxstruc_arr(1).data.data,1);   % A number of scenarios
I=size(toolboxstruc_arr(1).data.data,2);     % A number of variables

%Find redundant constraints:
ic=[];
    
for j=2:J
    for j1=1:j-1       
         if ubound_sd(j)>=ubound_sd(j1) && benchmark_sd(j)<=benchmark_sd(j1), ic=[ic,j]; break, end
         if ubound_sd(j)<=ubound_sd(j1) && benchmark_sd(j)>=benchmark_sd(j1), ic=[ic,j1]; end
    end   
end

ubound_sd_loc=ubound_sd;
benchmark_sd_loc=benchmark_sd;

ubound_sd_loc(ic) = [];
benchmark_sd_loc(ic) = [];

%Create problem statement:
problem_head=['Problem_ssd, maximize', char(10),...
              '  linear(matrix_sde)', char(10),...
              'Constraint: = 1', char(10),...
              '  linear(matrix_budget)'];

problem_newc='';

problem_newc=[problem_newc ...
              'MultiConstraint: <= vector_ub',char(10),...
              '  pm_pen(vector_th, matrix_sde)',char(10)];

problem_box=[ 'Box: >=0, <=1',char(10),...
              'Solver: precision = 9'];              
              
problem_statement=[problem_head,char(10), problem_newc, problem_box];

disp(sprintf('Number of non-redundand constraints %g',size(ic,2)));

clear problem_head problem_newc problem_box j j1 ic;

%Pack vector with upper bounds in constraints to PSG Toolbox structure:
toolboxstruc_arr(3) = tbpsg_vector_pack('vector_ub', ubound_sd_loc);

%Pack vector with parameters of pm_pen function to PSG Toolbox structure:
toolboxstruc_arr(4) = tbpsg_vector_pack('vector_th', -benchmark_sd_loc);

% Uncomment the following line to open the problem in Toolbox Window
% tbpsg_toolbox(problem_statement,toolboxstruc_arr);

%Optimize problem:
[solution_SSD, outargstruc_SSD] = tbpsg_run(problem_statement, toolboxstruc_arr);

%Display main results:

disp(' '); disp(' ')
disp(' Report ')
disp(' ')
disp('Solution_SSD ')
dx = strfind(solution_SSD, 'Constraint:');
disp(solution_SSD(1:dx(1)-1));
clear dx;
Optimal_point = tbpsg_optimal_point_data(solution_SSD, outargstruc_SSD)';
disp('Optimal point:');
for i=1:I;  disp(['x', int2str(i),' = '  num2str(Optimal_point(i),7)]);
end
clear i; disp(' ')

%=======================================================================|
%American Optimal Decisions, Inc. Copyright                             |
%Copyright American Optimal Decisions, Inc. 2007-2016.                 |
%American Optimal Decisions (AOD) retains copyrights to this material.  |
%                                                                       |
%Permission to reproduce this document and to prepare derivative works  |
%from this document for internal use is granted, provided the copyright |
%and No Warranty  statements are included with all reproductions      |
%and derivative works.                                                  |
%                                                                       |
%For information regarding external or commercial use of copyrighted    |
%materials owned by AOD, contact AOD at support@aorda.com.              |
%=======================================================================|